import cv2 as cv
import numpy as np
import tkinter as tk
from tkinter import filedialog, messagebox
import os
from PIL import Image, ImageTk
import configparser
from ImageIO import ImageIO

class Commands:
    def __init__(self, gui, shared_data):
        self.gui = gui
        self.shared_data = shared_data
        self.io = ImageIO(shared_data)

    def load_left_image(self):
        filepath = filedialog.askopenfilename(title="Select Left Image")
        if filepath:
            self.shared_data.left_image = self.io.load_image(filepath)
            print("Left image loaded.")

    def load_right_image(self):
        filepath = filedialog.askopenfilename(title="Select Right Image")
        if filepath:
            self.shared_data.right_image = self.io.load_image(filepath)
            print("Right image loaded.")

    def smooth_gradient(self,length, rate):
        # Create a linear gradient that transitions from full brightness (1) to zero (0)
        x = np.linspace(0, 1, length)
        # Apply a non-linear function (e.g., quadratic) for a smoother transition
        gradient = 1 - (x ** 2)  # Quadratic falloff to create a smooth transition
        gradient = np.clip(gradient * rate, 0, 1)  # Scale the gradient by the rate
        return gradient

    def show_left_image(self):
        if self.shared_data.left_image is not None:
            self.shared_data.isLeft = True
            left = self.shared_data.left_image.copy()
            height = left.shape[0]
            left_overlap_start = left.shape[1] - self.shared_data.overlap_region
            
            # Create a smooth gradient alpha mask for the left image (1 to 0)
            gradient_alpha = np.linspace(1, 0, self.shared_data.overlap_region)
            gradient_alpha = np.tile(gradient_alpha, (height, 1))

            # Apply the alpha gradient to the overlapping region
            left_overlap = left[:, left_overlap_start:, :].copy()
            left_overlap = (left_overlap * gradient_alpha[:, :, np.newaxis]).astype(np.uint8)

            # Replace the modified overlapping region back into the original left image
            left[:, left_overlap_start:, :] = left_overlap

            # Show the modified image
            self.io.show_image(left, canvas=self.gui.canvas)
            print("Left image displayed.")
        else:
            messagebox.showerror("Error", "No left image loaded!")

    def show_right_image(self):
        if self.shared_data.right_image is not None:
            self.shared_data.isLeft = False
            right = self.shared_data.right_image.copy()
            height = right.shape[0]
            right_overlap_end = self.shared_data.overlap_region
            
            # Create a smooth gradient alpha mask for the right image (0 to 1)
            gradient_alpha = np.linspace(0, 1, right_overlap_end)
            gradient_alpha = np.tile(gradient_alpha, (height, 1))

            # Apply the alpha gradient to the overlapping region
            right_overlap = right[:, :right_overlap_end, :].copy()
            right_overlap = (right_overlap * gradient_alpha[:, :, np.newaxis]).astype(np.uint8)

            # Replace the modified overlapping region back into the original right image
            right[:, :right_overlap_end, :] = right_overlap

            # Show the modified image
            self.io.show_image(right, canvas=self.gui.canvas)
            print("Right image displayed.")
        else:
            messagebox.showerror("Error", "No right image loaded!")
        
    def show_right_original(self):
        # Show original image
        if self.shared_data.right_image is not None:
            self.shared_data.isLeft = False
            right = self.shared_data.right_image.copy()
            self.io.show_image(right, canvas=self.gui.canvas)
            print("Right image displayed.")
        else:
            messagebox.showerror("Error", "No right image loaded!")

    def show_left_original(self):
        # Show original image
        if self.shared_data.left_image is not None:
            self.shared_data.isLeft = True
            left = self.shared_data.left_image.copy()
            self.io.show_image(left, canvas=self.gui.canvas)
            print("Left image displayed.")
        else:
            messagebox.showerror("Error", "No left image loaded!")


    def set_overlap_region(self):
        self.shared_data.overlap_region = self.gui.overlap.get()
        print(f"Overlap region set to {self.shared_data.overlap_region}")
        if(self.shared_data.isLeft):
            self.show_left_image()
        else:
            self.show_right_image()

    def increase_overlap(self):
        current_overlap = self.gui.overlap.get()
        self.gui.overlap.set(current_overlap + 1)
        self.set_overlap_region()
        if(self.shared_data.isLeft):
            self.show_left_image()
        else:
            self.show_right_image()
        
        
        

    def decrease_overlap(self):
        current_overlap = self.gui.overlap.get()
        self.gui.overlap.set(max(0, current_overlap - 1))  # Prevents negative overlap
        self.set_overlap_region()
        if(self.shared_data.isLeft):
            self.show_left_image()
        else:
            self.show_right_image()

